
--------------------------------------
----  M e a d e r  T a c t i c s  ----
--------------------------------------
--[[

* Created by: meadermad at gmail dot com
* Submit bug reports by e-mail or at curse.com/ (no adress yet)
* Version: 0.6.1
* Updated: 15.04.2015
Are you tired of PUG-raids where no one knows the tactics ant you are the one that has to type your fingers sore to explain it all? Then this is the addon for you! Explain the fights with just a few clicks...

== Current features ==
* Three sets of tactics: Normal, Heroic and Mythic.
* You easily can create additional sets.
* Insert raid-icons to messages.
* Ordering messages by moving them up and down in the list.
* Only supports raids from WoD.
* Use slash command '/mead test' to sends messages to 'SAY' instead of 'RAID', for testing purposes.
* Movable frames that remember their positions between sessions and accross characters.

**Please note that the tactic-sets are empty, so you will have to write up the tactic messages before the addon is usable.**

== How to get started ==
* Install the addon like any other. The "MeaderTactics" folder should be in your "AddOns" folder.
* Use the slash command "/mead" to open the panel.
* Click the edit button to enter edit mode.
* Set the tactic-set you want to edit or create a new one (Top button where its says "Normal").
* Click the raid you want to edit.
* Click the boss you want to edit.
* Click "Add new.." to add the first message.
* You'll get it from there...

== How to use ==
* Slash command "/mead"
* Select the right tactic-set (Normal, Heroic, etc...)
* Click the raid you are in.
* Click the boss you are at.
* That is all!

--------------------
---- Change Log ----
--------------------

* Added several features to the edit box: Move message up and down, delete, insert raid icons and boss link.
* Cleaned up the layout, textures and buttons for a much smoother feel.
* Re-organized the whole database with all three main tables now grouped together under one table.
* Re-wrote most of the button generating codes to suit the new DB system.
* If player is not in an instance group, messages are sent to "SAY", for testing purposes.

-------------------
---- ToDo List ----
-------------------
* Move the database init functions and database constructors to a separate file: meaderDB.lua. This set of code is only needed in the install prosess.
* Add functionality to import and export tactic-sets and messages.
* The main frame does not hide with escape key.
* If a raid button or boss button is clicked while editBox is open, the box still is there and some weird shit happens when you click save. (Nothing dangerous, it saves at the right spot). Guess is is a hideButtons("message"... error.
* Add sounds for button clicks.
* Warning before deleting
* Warning before installing and wiping the DB
* Rightclick on messages in edit mode should delete (after warning). Say so in tooltip.
* Adding an options panel. Set what channels to shout.
* Posibility to choose for each message what channel it should be sent to. ie. RAID or RAID_WARNING ..
* Get the journal links going. for now only the boss has one... for the highmaul bosses. Blackrock bosses does not have any encounterID's...
* Editbox. get it to select all text when opens.
* Expand the use of XML templates.
* Option to enter testMode, where messages are sent to "SAY" instead of "RAID"
* Auto-detect what raid (or even boss) you are at and show a small "Explain it" button.
--]]

-----------------------------
---- I N I T I A L I Z E ----
-----------------------------

-- A few variables used by many functions
local difficultyID = 1
local editMode = false
local testMode = false
--etc

--Set the main object 'f'. Used for frames and functions
local f = {}
local meaderDB
local setting

--Definge the array to store each button. Sorted into three tables one for each level of buttons. A raid button is referred to as 'f.button["raid"][order]' where order is the number of buttons from top and down.
f.button = {["raid"]={},["boss"]={},["message"]={}}
f.eButton = {[1]={},[2]={},[3]={},[4]={},[5]={}}


--Create the main frame
f.frame = CreateFrame("Frame", "MeaderTactics_MainFrame", UIParent) 
f.mainFrameDefined = false

--Predefine functions
local init
local install
local mainFrame
local mainFrameButtons
local listButtons
local createButton
local hideButtons
local hideButtonsLoop
local printMessages
local createEditPanelFrame
local sortMessages
local countMessages
local deleteMessage
local moveMessage
local createDifficultyDropDown
local addNewDifficulty
local savePositions
local editPanelHelper
local listEncounterButtons
local createEncounterButton
local showTooltip

-- Define the global meaderFrame. Set as 'f'
--meaderFrame = f


---------------------------
---- F U N C T I O N S ----
---------------------------

init = function()
	--Load saved variables into the local DB
	meaderDB = meaderTacticsDB
	setting = meaderTacticsSetting
end -- init

install = function()
meaderDB = {
	["difficulty"] = {
		"Normal", -- [1]
		"Heroic", -- [2]
		"Mythic", -- [3]
	},
	["boss"] = {
		{
			["order"] = 1,
			["description"] = "",
			["parentID"] = 1,
			["ID"] = 1128,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Kargath",
		}, -- [1]
		{
			["order"] = 2,
			["description"] = "",
			["parentID"] = 1,
			["ID"] = 971,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "The Butcher",
		}, -- [2]
		{
			["order"] = 3,
			["description"] = "",
			["parentID"] = 1,
			["ID"] = 1196,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Brackenspore",
		}, -- [3]
		{
			["order"] = 4,
			["description"] = "",
			["parentID"] = 1,
			["ID"] = 1195,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Tectus",
		}, -- [4]
		{
			["order"] = 5,
			["description"] = "",
			["parentID"] = 1,
			["ID"] = 1148,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Twin Orgon",
		}, -- [5]
		{
			["order"] = 6,
			["description"] = "",
			["parentID"] = 1,
			["ID"] = 1153,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Ko'ragh",
		}, -- [6]
		{
			["order"] = 7,
			["description"] = "",
			["parentID"] = 1,
			["ID"] = 1197,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Imperator Mar'gok",
		}, -- [7]
		{
			["order"] = 1,
			["description"] = "",
			["parentID"] = 2,
			["ID"] = 0,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Oregorger",
		}, -- [8]
		{
			["order"] = 2,
			["description"] = "",
			["parentID"] = 2,
			["ID"] = 0,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Gruul",
		}, -- [9]
		{
			["order"] = 3,
			["description"] = "",
			["parentID"] = 2,
			["ID"] = 0,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Blast Furnace",
		}, -- [10]
		{
			["order"] = 4,
			["description"] = "",
			["parentID"] = 2,
			["ID"] = 0,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Hans'gar and Franzok",
		}, -- [11]
		{
			["order"] = 5,
			["description"] = "",
			["parentID"] = 2,
			["ID"] = 0,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Flamebender Ka'graz",
		}, -- [12]
		{
			["order"] = 6,
			["description"] = "",
			["parentID"] = 2,
			["ID"] = 0,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Kromog",
		}, -- [13]
		{
			["order"] = 7,
			["description"] = "",
			["parentID"] = 2,
			["ID"] = 0,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Beastlord Darmac",
		}, -- [13]
		{
			["order"] = 8,
			["description"] = "",
			["parentID"] = 2,
			["ID"] = 0,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Operator Thogar",
		}, -- [14]
		{
			["order"] = 9,
			["description"] = "",
			["parentID"] = 2,
			["ID"] = 0,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Iron Maidens",
		}, -- [15]
		{
			["order"] = 10,
			["description"] = "",
			["parentID"] = 2,
			["ID"] = 0,
			["journalLink"] = "",
			["icon"] = "",
			["name"] = "Blackhand",
		}, -- [16]
	},
	["message"] = {
		{
			["name"] = "Please edit before using...",
			["difficultyID"] = 1,
			["order"] = 1,
			["parentID"] = 1,
		}, -- [1]
	},
	["raid"] = {
		{
			["expansion"] = 6,
			["name"] = "Highmaul",
			["order"] = 1,
			["ID"] = 477,
			["journalLink"] = "",
			["icon"] = "",
			["description"] = "",
		}, -- [1]
		{
			["expansion"] = 6,
			["name"] = "Blackrock Foundry",
			["order"] = 2,
			["ID"] = 457,
			["journalLink"] = "",
			["icon"] = "",
			["description"] = "",
		}, -- [2]
	},
	["expansion"] = {
		"Classic", -- [1]
		"The Burning Crusades", -- [2]
		"Wrath of the Lich King", -- [3]
		"Cataclysm", -- [4]
		"Mists of Pandaria", -- [5]
		"Warlords of Draenor", -- [6]
	}
}
setting = {
		["buttonHeight"] = 30,
		["currentDifficultyID"] = 1,
		["mainFrameX"] = 0,
		["editPanelPointA"] = "CENTER",
		["editPanelPointX"] = 50,
		["buttonTemplate"] = "MeaderTactics_Button_Template", --"UIMenuButtonStretchTemplate", 
		["sendToChannels"] = {
			"RAID", -- [1]
		},
		["mainFrameY"] = 0,
		["editPanelPointY"] = 50,
		["buttonWidth"] = 200,
		["editPanelHeight"] = 150,
		["editPanelWidth"] = 300,
		["mainFrameA"] = "CENTER",
		["editBoxHeight"] = 150,
		["editBoxWidth"] = 300,
		["diffDropdownButtonTemplate"] = "MeaderTactics_Button_Template",  --"UIMenuButtonStretchTemplate",
		["editPanelBackdropBgFile"] = "Interface\DialogFrame\UI-DialogBox-Background-Dark",
		["editPanelEdgeFile"] = "Interface\DialogFrame\UI-DialogBox-Border"
	}

	meaderTacticsDB = meaderDB
	meaderTacticsSetting = setting
end -- install

mainFrame = function()
		if f.mainFrameDefined == true then
	   		f.frame:SetPoint(setting.mainFrameA, setting.mainFrameX, setting.mainFrameY)
			f.frame:Show()
			return
		end
		
		--Define the main frame
   		f.frame:SetMovable(true)
   		f.frame:EnableMouse(true)
   		f.frame:RegisterForDrag("LeftButton")
   		f.frame:SetScript("OnDragStart", f.frame.StartMoving)
   		f.frame:SetScript("OnDragStop", f.frame.StopMovingOrSizing)
   		f.frame:SetResizable(true)
   		f.frame:SetMinResize(100, 100)
   		f.frame:SetPoint(setting.mainFrameA, setting.mainFrameX, setting.mainFrameY)
   		f.frame:SetSize(200,50)
   		f.header = f.frame:CreateFontString()
		f.header:SetFont(STANDARD_TEXT_FONT,15, "OUTLINE")
		f.header:SetJustifyH("CENTER")
		f.header:SetPoint("TOPLEFT", f.frame,"TOPLEFT",0,0)
		f.header:SetSize(200,25)
		f.header:SetTextColor(1, 0.3, 0, 1)    --light blue for original buttons: (0, 0.6, 1, 1) ---nice orange:(0.9, 0.7, 0, 1)
		f.header:SetText("Meader Tactics")
   		mainFrameButtons()
   		f.mainFrameDefined = true
   		f.frame:SetScript("OnLoad", function(self) tinsert(UISpecialFrames, self:GetName()) end)
   		f.frame:SetScript("OnShow", function(self) PlaySound("igCharacterInfoOpen") end)
   		f.frame:SetScript("OnHide", function(self)
   			savePositions()
   			if f.editPanel and f.editPanel:IsShown() then f.editPanel:Hide() end
   			PlaySound("igCharacterInfoClose")
   		end)

end


mainFrameButtons = function()
	--[[ToolTip
	f.toolTip = CreateFrame("GameTooltip","meaderToolTip", f.frame, "GameTooltipTemplate")
	f.toolTip:SetOwner(f.frame, "ANCHOR_CURSOR")
	--]]

	--Close main frame button
	local closeButton = CreateFrame("Button", "MEaderTacticsCloseAllButton", f.frame, "UIPanelCloseButton")
   	closeButton:SetScript("OnClick", function(self)
         print("Go kill some bosses!")
         HideParentPanel(self)
    end)
   	closeButton:SetPoint("BOTTOMLEFT", -3, 0)
    f.closeButton = closeButton
    
   --Difficulty Button (shows a dropdown)
   local normalButton   
   normalButton = CreateFrame("Button", "MeaderTacticsDifficultyDropDownMenuButton", f.frame, setting.diffDropdownButtonTemplate)   
   normalButton:SetPoint("BOTTOMRIGHT", -50, 5)
   normalButton:SetHeight(20)
   normalButton:SetWidth(125)
   normalButton:SetText(meaderDB["difficulty"][difficultyID])
   f.difficultyButton = normalButton    

	f.difficultyDropDownFrame = CreateFrame("Frame", "meaderTacticts_difficultyDropDown", f.frame)
	f.difficultyDropDownFrame:SetPoint("TOPRIGHT", f.frame, "BOTTOMRIGHT", -50, 5)
	f.difficultyDropDownFrame:SetResizable(true)
   	f.difficultyDropDownFrame:SetMinResize(100, 100)
	f.difficultyDropDownFrame:SetSize(125,20*(#meaderDB["difficulty"]+3))
	--local texture = f.difficultyDropDownFrame:CreateTexture("ARTWORK")
	--texture:SetAllPoints()
	--texture:SetTexture(0,0,0,1)
	f.difficultyDropDownFrame.button = {}
   createDifficultyDropDown()
   f.difficultyDropDownFrame:Hide()
   
   normalButton:SetScript("OnClick", function(self)
         if f.difficultyDropDownFrame:IsShown() then
            f.difficultyDropDownFrame:Hide()
            listButtons("raid",0)
            self:UnlockHighlight()
         else
            f.difficultyDropDownFrame:Show()
            hideButtons("raid")
            self:LockHighlight()
         end
   end)
   normalButton:SetScript("OnEnter", function(self)
   		local line1 = "Change raid difficulty"
   		local line2 = "You can also create more sets,"
   		local line3 = "for different tactics."
		showTooltip(self, line1, line2)
   end)
   normalButton:SetScript("OnLeave", function(self) 
   		GameTooltip:Hide()
   end)

   
   -- EditMode button
   local editButton = CreateFrame("Button", "MeaderTacticsEditModeButton", f.frame, setting.buttonTemplate)--"UIPanelButtonTemplate")
   editButton:SetScript("OnClick", function(self)
         if editMode then
            editMode=false
            editButton:UnlockHighlight()
         else
            editMode=true
            editButton:LockHighlight()
         end
         listButtons("raid",0)
	end)
   
   editButton:SetPoint("BOTTOMRIGHT", 0, 5)
   editButton:SetHeight(20)
   editButton:SetWidth(50)
   editButton:SetText("Edit")
   if editMode then editButton:LockHighlight() else editButton:UnlockHighlight() end
   
   --ToolTip
   editButton:SetScript("OnEnter", function(self)
   		local line1
   		local line2
   		if editMode then line1="Edit mode is ON";line2="Click to enter normal mode." else line1="Edit mode is OFF";line2="Click to enter edit mode." end
		showTooltip(self, line1, line2)
   end)
   editButton:SetScript("OnLeave", function(self) 
   		GameTooltip:Hide()
   end)
   f.editButton = editButton
end -- mainFrameButtons

listButtons = function(tableName, parentID)
--DEBUG
--print ("listButtons tableName: "..tableName.." parentID: "..parentID)
--END DEBUG
	local prevOrder = 1
	local text = ""
	hideButtons(tableName) 
	for k,v in pairs(meaderDB[tableName]) do
		if tableName == "raid" then
			createButton(tableName, k, meaderDB[tableName][k]["order"], parentID, false)
		elseif tableName == "boss" and meaderDB[tableName][k]["parentID"] == parentID then
			createButton(tableName, k, meaderDB[tableName][k]["order"], parentID, false)		
		elseif tableName == "message" and meaderDB[tableName][k]["parentID"] == parentID and meaderDB[tableName][k]["difficultyID"] == difficultyID then
			createButton(tableName, k, meaderDB[tableName][k]["order"], parentID, false)
			prevOrder = prevOrder + 1
		end
	end
	if tableName == "message" then
		--Add a "Add new.." button at the bottom
		createButton(tableName, #meaderDB["message"]+1, prevOrder, parentID, true)
	end
end --listButtons

createButton = function(tableName, tableIndex, order, parentID, addNew)
	local buttonID = tableName..order
	local posY
	local text
	local shortText
	if tableName == "raid" then posY = 0 elseif tableName == "boss" then posY = setting.buttonWidth else posY = 2 * setting.buttonWidth end
	
	
	if f.button[tableName][order] then
		f.button[tableName][order]:Show()
	else
		--Create a new button
		f.button[tableName][order] = CreateFrame("Button", "MeaderTacticsButton"..buttonID, f.frame, setting.buttonTemplate)
		f.button[tableName][order]:SetPoint("BOTTOMLEFT", posY, -(setting.buttonHeight * (order)))
		f.button[tableName][order]:SetSize(setting.buttonWidth, setting.buttonHeight)
		f.button[tableName][order]:Show()
	end

	if not addNew then
		text = meaderDB[tableName][tableIndex]["name"]
		if strlen(text) > 25 then shortText = strsub(text,1,25).."..." else shortText = text end
	else
		text = "Add new.."
		shortText = text
	end
	
	
	f.button[tableName][order]:SetText(shortText)
	f.button[tableName][order]:UnlockHighlight()

	if tableName == "raid" then
		f.button[tableName][order]:SetScript("OnClick", function(self, button, down)
			unlockButtonsLoop("raid")
			self:LockHighlight()

			listButtons("boss", tableIndex)
		end)
	elseif tableName == "boss" and editMode then
		f.button[tableName][order]:SetScript("OnClick", function(self, button, down)
			unlockButtonsLoop("boss")
			self:LockHighlight()
			listButtons("message", tableIndex)
		end)
	elseif tableName == "boss" and not editMode then
		f.button[tableName][order]:SetScript("OnClick", function(self, button, down)
		printMessages(tableIndex)
		end)		
	elseif tableName == "message" and editMode then
		f.button[tableName][order]:SetScript("OnClick", function(self, button, down)
			unlockButtonsLoop("message")
			self:LockHighlight()
			createEditPanelFrame(tableIndex, text, order, parentID, addNew)
		end)	
	end
end --createButton

createDifficultyDropDown = function()
	local pos = 1
	for k,v in ipairs(meaderDB["difficulty"]) do
		if not f.difficultyDropDownFrame.button[pos] then
			--Create a new button
			f.difficultyDropDownFrame.button[pos] = CreateFrame("Button", "MeaderTacticsDifficultyDropDownOptionButton"..pos, f.difficultyDropDownFrame, setting.diffDropdownButtonTemplate)
			f.difficultyDropDownFrame.button[pos]:SetPoint("TOPLEFT", 0, -(pos*20)+20)
			f.difficultyDropDownFrame.button[pos]:SetSize(125,20)
		else
			f.difficultyDropDownFrame.button[pos]:Show()
		end
		f.difficultyDropDownFrame.button[pos]:SetText(v)
		f.difficultyDropDownFrame.button[pos]:SetScript("OnClick", function(self)
			difficultyID = k
			--hideButtons("message")
			listButtons("raid",0)
			f.difficultyButton:SetText(v)
			f.difficultyButton:UnlockHighlight()
			f.difficultyDropDownFrame:Hide()
		end)
		pos = pos + 1
	end--for
	-- Add a "Add new" button at the end of the dropdown
	if not f.difficultyDropDownFrame.button[pos] then
		--Create a new button
		f.difficultyDropDownFrame.button[pos] = CreateFrame("Button", "MeaderTacticsDifficultyDropDownOptionButton"..pos, f.difficultyDropDownFrame, setting.diffDropdownButtonTemplate)
		f.difficultyDropDownFrame.button[pos]:SetPoint("TOPLEFT", 0, -(pos*20)+20)
		f.difficultyDropDownFrame.button[pos]:SetSize(130,20)
	else
		f.difficultyDropDownFrame.button[pos]:Show()
	end
	f.difficultyDropDownFrame.button[pos]:SetText("Add new")
	f.difficultyDropDownFrame.button[pos]:SetScript("OnClick", function(self)
		--print("Add new difficulty will be developed later on...")
		addNewDifficulty(pos)
	end)

end--createDifficultyDropDown

addNewDifficulty = function(pos)
	if not f.addNewDifficulty then
		f.addNewDifficulty = CreateFrame("EditBox", "MeaderTacticsAddNewDifficultyEditBox", f.frame, "InputBoxTemplate")
		f.addNewDifficulty:SetPoint("TOPLEFT",f.difficultyDropDownFrame,"TOPLEFT", 0, -((pos+1)*20)+20)
		f.addNewDifficulty:SetSize(130, 30)
	else
		f.addNewDifficulty:Show()
		f.addNewDifficulty:SetPoint("TOPLEFT",f.difficultyDropDownFrame,"TOPLEFT", 0, -((pos+1)*20)+20)
	end
	f.addNewDifficulty:SetText("New difficulty title...")
	f.addNewDifficulty:SetScript("OnEnterPressed", function(self)
		meaderDB["difficulty"][pos]=self:GetText()
		createDifficultyDropDown()
		self:Hide()
	end)
	f.addNewDifficulty:SetScript("OnEscapePressed", function(self) self:Hide() end)

	
end--addNewDifficulty

hideButtons = function(cTable)
	if cTable == "raid" then hideButtonsLoop("message"); hideButtonsLoop("boss"); hideButtonsLoop("raid")
	elseif cTable == "boss" then hideButtonsLoop("message"); hideButtonsLoop("boss")
	elseif cTable == "message" then hideButtonsLoop("message")
	end
end --hideButtons


hideButtonsLoop = function(tableName)
	for k,v in pairs(f.button[tableName]) do
			f.button[tableName][k]:Hide()
	end
end -- hideButtonsLoop

unlockButtonsLoop = function(tableName)
	for k,v in pairs(f.button[tableName]) do
		f.button[tableName][k]:UnlockHighlight()
	end
end --unlockButtonsLoop

printMessages = function(tableIndex)
	sortedTable = sortMessages(tableIndex)
	for k,v in pairs(sortedTable) do
		if testMode then  --or not IsInGroup(LE_PARTY_CATEGORY_INSTANCE)
			SendChatMessage(meaderDB["message"][v]["name"] ,'SAY')
		else
			SendChatMessage(meaderDB["message"][v]["name"] ,'RAID')
		end
	end--for
end

sortMessages = function(tableIndex)
	sortedTable = {}
	for k,v in pairs(meaderDB["message"]) do
		if meaderDB["message"][k]["parentID"] == tableIndex and meaderDB["message"][k]["difficultyID"] == difficultyID then
			sortedTable[meaderDB["message"][k]["order"]] = k
		end
	end--for
	return sortedTable
end--sortMessages

deleteMessage = function (tableIndex)
	--print("meader Tactics: Not yet implemented... Can't delete message #"..tableIndex)
	local order = meaderDB["message"][tableIndex]["order"]
	local parent = meaderDB["message"][tableIndex]["parentID"]
	table.remove(meaderDB["message"], tableIndex)
	for k,v in pairs(meaderDB["message"]) do
		if meaderDB["message"][k]["order"] > order and meaderDB["message"][k]["parentID"]==parent then
			meaderDB["message"][k]["order"] = meaderDB["message"][k]["order"]-1
		end
	end
	print("meaderTactics: Deleted message #"..tableIndex.."")
end--deleteMessage

moveMessage = function (direction, tableIndex)
	local order = meaderDB["message"][tableIndex]["order"]
	local newOrder
	local target
	local parent = meaderDB["message"][tableIndex]["parentID"]
	
	if direction == "up" then
		newOrder = order-1
	elseif direction == "down" then
		newOrder = order+1
	end
	for k,v in pairs(meaderDB["message"]) do
		if (meaderDB["message"][k]["order"] == newOrder) and (meaderDB["message"][k]["parentID"] == parent) then --and (not k==tableIndex) then
			target = k
		end
	end
	meaderDB["message"][tableIndex]["order"] = newOrder
	meaderDB["message"][target]["order"] = order
end--moveMessage

countMessages = function (tableIndex)
	local cnt = 0
	for k,v in pairs(meaderDB["message"]) do
		if meaderDB["message"][k]["parentID"] == tableIndex then cnt = cnt +1 end
	end
	return cnt
end--countMessages

-----------------
---- TOOLTIP ----
-----------------

--[[ Basic use:
   f:SetScript("OnEnter", function(self)
   		local line1 = "title in gold"
   		local line2 = "second line in white"
		showTooltip(self, line1, line2)
   end)
   f:SetScript("OnLeave", function(self) 
   		GameTooltip:Hide()
   end)
--]]
showTooltip = function(self, ...)
	local line1, line2, line3, line4, line5, line6, line7, line8, line9 = ...
   	GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
 	GameTooltip:SetText(line1)
 	if line2 then GameTooltip:AddLine(line2, 1,1,1) end
 	if line3 then GameTooltip:AddLine(line3, 1,1,1) end
 	if line4 then GameTooltip:AddLine(line4, 1,1,1) end
 	if line5 then GameTooltip:AddLine(line5, 1,1,1) end
 	if line6 then GameTooltip:AddLine(line6, 1,1,1) end
 	if line7 then GameTooltip:AddLine(line7, 1,1,1) end
 	if line8 then GameTooltip:AddLine(line8, 1,1,1) end
 	if line9 then GameTooltip:AddLine(line9, 1,1,1) end
 	GameTooltip:Show()

end--showTooltip


----------------------------
---- E D I T  P A N E L ----
----------------------------

createEditPanelFrame = function(tableIndex, text, order, parentID, addNew)
	if not f.editPanel then
		f.editPanel = CreateFrame("Frame", "MeaderTacticsEditPanelMainFrame", f.frame)
		f.editPanel:SetSize(setting.editPanelWidth, setting.editPanelHeight)
		f.editPanel:SetPoint(setting.editPanelPointA, UIParent,setting.editPanelPointA, setting.editPanelPointX, setting.editPanelPointY)
		f.editPanel:SetMovable(true)
   		f.editPanel:EnableMouse(true)
   		f.editPanel:RegisterForDrag("LeftButton")
   		f.editPanel:SetScript("OnDragStart", f.editPanel.StartMoving)
   		f.editPanel:SetScript("OnDragStop", f.editPanel.StopMovingOrSizing)
   		f.editPanel:SetScript("OnHide", function(self) savePositions() end)

		local texture = f.editPanel:CreateTexture("BACKGROUND")
		texture:SetAllPoints()
		texture:SetTexture(0,0,0,0)
		local backdrop = {
  			bgFile = "Interface/DialogFrame/UI-DialogBox-Background", --setting.editPanelBackdropBgFile,  
  			edgeFile = "Interface/Tooltips/UI-Tooltip-Border", --setting.editPanelEdgeFile,
  			tile = true,
  			tileSize = 32,
  			edgeSize = 15,
  			insets = {
    			left = 4,
    			right = 4,
    			top = 3,
    			bottom = 3
  			}
		}--]]
		f.editPanel:SetBackdrop(backdrop)
		--f.editPanel:SetBackdropColor(0,0,0,1)
	else
		f.editPanel:Show()
	end

	if not f.editBox then
   		f.editBox = CreateFrame("EditBox", "MeaderTacticsEditBox", f.editPanel) --, "InputBoxTemplate") --"InputScrollFrameTemplate")
   		f.editBox:SetHeight(150)
   		f.editBox:SetWidth(300)
   		f.editBox:SetPoint("TOPLEFT", f.editPanel, "TOPLEFT", 0, 0)
   		f.editBox:SetMaxLetters(255)
   		f.editBox:SetMultiLine(true)
		f.editBox:SetTextInsets(20, 20, 20, 20)   
		f.editBox:SetTextColor(1, 1, 1, 1) 
   		f.editBox:EnableKeyboard(true)
   		f.editBox:SetFontObject(GameFontNormal)
   		f.editBox:SetAutoFocus(false)
   		f.editBox:SetFocus()

    else
   		f.editBox:Show()
    end
   	--Edit box scripts
   	f.editBox:SetScript("OnChar",function(self,userInput,...)
      	local text = self:GetText() or ""
	  	self: SetText(text)--..""..userInput)
	end)
	if addNew then
   		f.editBox:SetScript( "OnEnterPressed", function (self) 
   			meaderDB["message"][tableIndex] = {name=self:GetText(),difficultyID=difficultyID,order=order,parentID=parentID}
   			hideButtons("message")
   			listButtons("message", parentID)
   			f.editPanel:Hide()
   		end)
	else
   		f.editBox:SetScript( "OnEnterPressed", function (self) 
   			meaderDB["message"][tableIndex]["name"] = self:GetText()
   			hideButtons("message")
   			listButtons("message", parentID)
   			f.editPanel:Hide()   		
   		end)
   	end
   	f.editBox:SetScript( "OnEscapePressed", function (self)
   		f.editBox:ClearFocus()
   		--f.editPanel:Hide()
   	end)
   	
 	f.editBox:SetText(text)
   	f.editBox:SetFocus()
   						  
	if not f.cancelButton then
   		f.cancelButton = CreateFrame("Button", "meaderTactics_cancelMessage", f.editPanel, setting.buttonTemplate)
   		f.cancelButton:SetPoint("BOTTOMRIGHT", f.editPanel, "BOTTOMRIGHT", -5, 5)
   		f.cancelButton:SetSize(48,25)
   		f.cancelButton:SetText("Cancel")
   		f.cancelButton:SetScript("OnClick", function(self)
   		f.editPanel:Hide()
		end)
	else
		f.cancelButton:Show()
	end
	
	if not f.saveButton then
		f.saveButton = CreateFrame("Button", "meaderTactics_saveMessage", f.editPanel, setting.buttonTemplate)
   		f.saveButton:SetPoint("BOTTOMRIGHT", f.editPanel, "BOTTOMRIGHT", -53.5, 5)
   		f.saveButton:SetSize(48,25)
   		f.saveButton:SetText("Save")
	else
		f.saveButton:Show()
	end
	if addNew then
   		f.saveButton:SetScript( "OnClick", function (self) 
   			meaderDB["message"][tableIndex] = {name=f.editBox:GetText(),difficultyID=difficultyID,order=order,parentID=parentID}
   			hideButtons("message")
   			listButtons("message", parentID)
   			f.editPanel:Hide()
   		end)
	else
   		f.saveButton:SetScript( "OnClick", function (self) 
   			meaderDB["message"][tableIndex]["name"] = f.editBox:GetText()
   			hideButtons("message")
   			listButtons("message", parentID)
   			f.editPanel:Hide()   		
   		end)
   	end

	if not f.deleteButton then
		f.deleteButton = CreateFrame("Button", "meaderTactics_deleteMessage", f.editPanel, setting.buttonTemplate)
   		f.deleteButton:SetPoint("BOTTOMRIGHT", f.editPanel, "BOTTOMRIGHT", -102, 5)
   		f.deleteButton:SetSize(48,25)
   		f.deleteButton:SetText("Delete")
   	else
   		f.deleteButton:Show()
   	end
   		f.deleteButton:SetScript( "OnClick", function (self) 
			deleteMessage(tableIndex)
   			hideButtons("message")
   			listButtons("message", parentID)
   			f.editPanel:Hide()
   		end)
   	if addNew then f.deleteButton:Hide() end

	if not f.moveUpButton then
		f.moveUpButton = CreateFrame("Button", "meaderTactics_deleteMessage", f.editPanel, setting.buttonTemplate)
   		f.moveUpButton:SetPoint("BOTTOMRIGHT", f.editPanel, "BOTTOMRIGHT", -150, 5)
   		f.moveUpButton:SetSize(48,25)
   		f.moveUpButton:SetText("Up")
   	else
   		f.moveUpButton:Show()
   	end
   	f.moveUpButton:SetScript( "OnClick", function (self) 
		moveMessage("up", tableIndex)
   		hideButtons("message")
   		listButtons("message", parentID)
   		f.editPanel:Hide()
   	end)
   	f.moveUpButton:SetScript("OnEnter", function(self)
   		local line1 = "Move message up"
   		showTooltip(self, line1)
   	end)
   	f.moveUpButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
   	if order==1 or addNew then f.moveUpButton:Hide() end
   	

	if not f.moveDownButton then
		f.moveDownButton = CreateFrame("Button", "meaderTactics_deleteMessage", f.editPanel, setting.buttonTemplate)
   		f.moveDownButton:SetPoint("BOTTOMRIGHT", f.editPanel, "BOTTOMRIGHT", -198.5, 5)
   		f.moveDownButton:SetSize(48,25)
   		f.moveDownButton:SetText("Down")
   	else
   		f.moveDownButton:Show()
   	end
   	f.moveDownButton:SetScript( "OnClick", function (self) 
			moveMessage("down", tableIndex)
   			hideButtons("message")
   			listButtons("message", parentID)
   			f.editPanel:Hide()
   		end)
   	f.moveDownButton:SetScript("OnEnter", function(self)
   		local line1 = "Move message down"
   		showTooltip(self, line1)
   	end)
   	f.moveDownButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
   	if order >= countMessages(parentID) or addNew then f.moveDownButton:Hide() end
		
	
	--Initiate the helper panel
	editPanelHelper(meaderDB["boss"][parentID]["ID"])
	f.editPanelHelper:Hide()
	
   	if not f.moreButton then
   		f.moreButton = CreateFrame("Button", "MeaderTacticsEditPanelHelperButton", f.editPanel, setting.buttonTemplate)
   		f.moreButton:SetPoint("BOTTOMLEFT", f.editPanel, "BOTTOMLEFT", 5, 5)
		f.moreButton:SetSize(49,25)
		f.moreButton:SetText("More")
	else
		f.moreButton:Show()
	end
	if f.editPanelHelper:IsShown() then f.moreButton:LockHighlight() else f.moreButton:UnlockHighlight() end
	
	f.moreButton:SetScript("OnClick", function(self)
		if f.editPanelHelper:IsShown() then
			f.editPanelHelper:Hide()
			self:UnlockHighlight()
		else
			f.editPanelHelper:Show()
			self:LockHighlight()
		end
	end)
	f.moreButton:SetScript("OnEnter", function(self)
   		local line1
   		local line2
   		if f.editPanelHelper:IsShown() then
   			line1 = "Hide helper"
			showTooltip(self, line1)
   		else
   			line1 = "Show helper window"
   			line2 = "A tool to insert raid markers"
   			line3 = "and links for abilities and endcounters"
			showTooltip(self, line1, line2, line3)
   		end
   end)
   f.moreButton:SetScript("OnLeave", function(self) 
   		GameTooltip:Hide()
   end)

	
end--createEditPanelFrame

---------------------------
---- EDIT PANEL HELPER ----
---------------------------
editPanelHelper = function(encounterID)
	if not f.editPanelHelper then
		f.editPanelHelper = CreateFrame("Frame", "MeaderTacticsEditPanelMainFrame", f.editPanel)
		f.editPanelHelper:SetSize(setting.editPanelWidth, setting.editPanelHeight)
		f.editPanelHelper:SetPoint("TOPLEFT", f.editPanel, "BOTTOMLEFT", 0,-5)
		f.editPanelHelper:SetMovable(true)
   		f.editPanelHelper:EnableMouse(true)
   		f.editPanelHelper:RegisterForDrag("LeftButton")
   		f.editPanelHelper:SetScript("OnDragStart", f.editPanel.StartMoving)
   		f.editPanelHelper:SetScript("OnDragStop", f.editPanel.StopMovingOrSizing)
		local texture = f.editPanelHelper:CreateTexture("BACKGROUND")
		texture:SetAllPoints()
		texture:SetTexture(0,0,0,0)
		local backdrop = {
  			bgFile = "Interface/DialogFrame/UI-DialogBox-Background", --setting.editPanelBackdropBgFile,  
  			edgeFile = "Interface/Tooltips/UI-Tooltip-Border", --setting.editPanelEdgeFile,
  			tile = true,
  			tileSize = 32,
  			edgeSize = 15,
  			insets = {
    			left = 4,
    			right = 4,
    			top = 3,
    			bottom = 3
  			}
		}--]]
		f.editPanelHelper:SetBackdrop(backdrop)
		--f.editPanel:SetBackdropColor(0,0,0,1)
	end
		f.editPanelHelper:SetPoint("TOPLEFT", f.editPanel, "BOTTOMLEFT", 0,-5)
	
	-- World Marker Buttons
	if not f.editPanelWorldMarkers then
		f.editPanelWorldMarkers = {}
	
		--UI-RaidTargetingIcon_1.blp 
		local worldMarkers = {[8] = "{Skull}", [7] = "{Cross}", [6] = "{Square}", [2] = "{Circle}", [1] = "{Star}", [4] = "{Triangle}", [3] = "{Diamond}"}
		local pos = 0
		for k,v in pairs(worldMarkers) do
			f.editPanelWorldMarkers[k] = CreateFrame("Button", "MeaderTacticsWorldMarkerButton_"..k, f.editPanelHelper, setting.buttonTemplate)
			f.editPanelWorldMarkers[k]:SetSize(25,25)
			f.editPanelWorldMarkers[k]:SetPoint("TOPLEFT",5+(pos*25)+5,-5)
			f.editPanelWorldMarkers[k]:SetNormalTexture("Interface/TargetingFrame/UI-RaidTargetingIcon_"..k)
			f.editPanelWorldMarkers[k]:SetScript("OnClick", function(self) f.editBox:Insert(v)end)
	   		f.editPanelWorldMarkers[k]:SetScript("OnEnter", function(self)
   				local line1 = "Insert "..worldMarkers[k].." raid marker"
				showTooltip(self, line1)
   			end)
   			f.editPanelWorldMarkers[k]:SetScript("OnLeave", function(self) 
   				GameTooltip:Hide()
   			end)

			pos = pos + 1
		end--for
	end--if
	
	--Encounter Link Browser Button
	local name, description, eencounterID, firstSection, encounterLink = EJ_GetEncounterInfo(encounterID)
	if not f.encounterButton then
		f.encounterButton = CreateFrame("Button", "MeaderTacticsWorldMarkerButton_1", f.editPanelHelper, setting.buttonTemplate)
		f.encounterButton:SetSize(100,20)
		f.encounterButton:SetPoint("TOPLEFT",10, -30)
		f.encounterButton:RegisterForClicks("LeftButtonUp","RightButtonUp")
		--f.editPanelWorldMarkers[1]:SetNormalTexture("Interface/TargetingFrame/UI-RaidTargetingIcon_"..k)
	end
	f.encounterButton:SetText(name)
	f.encounterButton:SetScript("OnClick", function(self, button, down)
		if button == "LeftButton" then
			--listEncounterButtons(firstSection, 1)
		else
			f.editBox:Insert(encounterLink)
		end
	end)
	f.encounterButton:SetScript("OnEnter", function(self)
   		local line1 = "Journal link to "..name
   		local line2 = "Left mouse button to browse more links (To be developed)."
   		local line3 = "Right mouse button to insert link."
		showTooltip(self, line1, line2, line3)
   end)
   f.encounterButton:SetScript("OnLeave", function(self) 
   		GameTooltip:Hide()
   end)

end--editPanelHelper


savePositions = function()
	if f.editPanel then
	local point, relativeTo, relativePoint, xOffset, yOffset = f.editPanel:GetPoint(1)
	setting.editPanelPointY = yOffset
	setting.editPanelPointX = xOffset
	setting.editPanelPointA = point
	end
	if f.frame then
	local point, relativeTo, relativePoint, xOffset, yOffset = f.frame:GetPoint(1)
	setting.mainFrameA = point
	setting.mainFrameX = xOffset
	setting.mainFrameY = yOffset
	end
	
end--savePositions

---- Encounter Link Browser ----

listEncounterButtons = function(sectionID, lvl)
	local pos = 1
	while sectionID do
		local name, description, headerType, abilityIcon, displayInfo, siblingID, nextSectionID, fileredByDifficulty, sectionLink, startsOpen, flag1, flag2, flag3, flag4 = EJ_GetSectionInfo(sectionID)
		createEncounterButton(name, abilityIcon, sectionLink, nextSectionID, lvl, pos)
		pos = pos + 1
		if pos > 20 then sectionID=false else sectionID = siblingID end
		
	end
end--listEncounterButtons

createEncounterButton = function(name, abilityIcon, sectionLink, nextSectionID, lvl, pos)
	
	if not f.eButton[lvl][pos] then
		f.eButton[lvl][pos] = CreateFrame("Button", "MeaderTactsEncounterLinkBrowserButton"..lvl.."-"..pos, f.editPanelHelper, "UIMenuButtonStretchTemplate")
		f.eButton[lvl][pos]:SetPoint("TOPLEFT", -55+(lvl*100)-100,-10+(pos*20)-20)
		f.eButton[lvl][pos]:SetSize(100,20)
		f.eButton[lvl][pos]:RegisterForClicks("LeftButtonUp","RightButtonUp")
	end
	f.eButton[lvl][pos]:SetText(name)
	f.eButton[lvl][pos]:SetScript("OnClick",function(self,button)
		if button == "LeftButton" then
			listEncounterButtons(nextSectionID, lvl+1)
		else
			f.editBox:Insert(sectionLink)
		end
	end)
	f.eButton[lvl][pos]:SetScript("OnEnter", function(self)
   		local line1 = "Jornal link to "..name
   		local line2 = "Left mouse button to browse more links."
   		local line3 = "Right mouse button to insert link."
		showTooltip(self, line1, line2, line3)
   end)
   f.eButton[lvl][pos]:SetScript("OnLeave", function(self) 
   		GameTooltip:Hide()
   end)
end--createEncounterButton

--------------------
-- EVENT HANDLERS --
--------------------
f.frame:RegisterEvent("ADDON_LOADED")
f.frame:SetScript("OnEvent",
   function(self, event, ...) 
      local arg = ...
      if event == "ADDON_LOADED" and arg == "MeaderTactics" then
         --Check if globals exist and load them
         DEFAULT_CHAT_FRAME:AddMessage("meaderTactics loaded. To open the raid selection window type '/mead'.",1,1,0)
         if not _G["meaderTacticsDB"] then
            --Greet the user for the first time and copy the raid table into the global.
            DEFAULT_CHAT_FRAME:AddMessage("Your database of raids, bosses and tactics is empty. Please configure your tactics by entering edit mode. Open the tactics window, select a raid and then a boss and click 'Add new' to get started. Happy raiding and good luck!")
            install()
         end
         init()     
      end
	end)


--------------------
-- SLASH COMMANDS --
--------------------

--This command reinstalls the database. Use with care as it wipes all stored info.
SLASH_MEADERTACTICSINSTALL1 = "/meaderDBClean"
function SlashCmdList.MEADERTACTICSINSTALL(...)
	install()
end

SLASH_MEADERTACTICS1 = "/mead"
function SlashCmdList.MEADERTACTICS(...)
	arg1 = ...
	if arg1 == "test" then
		if testMode then testMode=false else testMode=true end
	else
		mainFrame()
		listButtons("raid", 0)
	end
end